@@ -33,8 +33,7 @@
 ##' @param cex.labels pairs() argument
 ##' @param font.labels pairs() argument
 ##' @param gap pairs() argument
-##' @param axes logical indicating whether axes are drawn (if g1 is not a
-##'	   function, they are omitted automatically)
+##' @param axes logical indicating whether axes are drawn
 ##' @param panel.border logical indicating whether a border is drawn around the
 ##'	   pairs (or not; to mimic the behavior of image())
 ##' @param key logical indicating whether a color key is drawn
@@ -73,7 +72,9 @@ pairs2 <- function(gcu.u,
 	      dc == c((n <- dc[1]),(d <- dc[2]), d), d>=2)
 
     panel <- match.fun(panel)
-    if(d * d > 500 - 3) stop("Currently, layout() only allows 500 different plot areas\n This limit was now reached. It might be extended in the future.")
+    if(d * d > 500 - 3 && getRversion() < "2.16.0")
+        stop("Currently, layout() only allows 500 different plot areas. This limit\n  ",
+             "is reached for d=",d,".  It will be extended for R 2.16.0 (and later).")
 
     ## preliminaries ###########################################################
 
